@@ -4,8 +4,8 @@
 #include <memory>
 #include <string>
 
-#include <cpr/api.h>
-#include <cpr/cpr.h>
+#include "cpr/api.h"
+#include "cpr/cpr.h"
 #include <curl/curl.h>
 
 #include "httpServer.hpp"
@@ -15,7 +15,7 @@ using namespace cpr;
 
 static HttpServer* server = new HttpServer();
 
-bool write_data(std::string /*data*/, intptr_t /*userdata*/) {
+bool write_data(std::string_view /*data*/, intptr_t /*userdata*/) {
     return true;
 }
 
@@ -92,12 +92,21 @@ TEST(MultiperformRemoveSessionTests, MultiperformRemoveMultipleSessionsTest) {
     }
 }
 
-TEST(MultiperformRemoveSessionTests, MultiperformRemoveNonExistingSessionTest) {
+TEST(MultiperformRemoveSessionTests, MultiperformRemoveNonExistingSessionEmptyTest) {
     std::shared_ptr<Session> session = std::make_shared<Session>();
     MultiPerform multiperform;
     EXPECT_THROW(multiperform.RemoveSession(session), std::invalid_argument);
 }
 
+TEST(MultiperformRemoveSessionTests, MultiperformRemoveNonExistingSessionTest) {
+    MultiPerform multiperform;
+    std::shared_ptr<Session> session = std::make_shared<Session>();
+    multiperform.AddSession(session);
+
+    std::shared_ptr<Session> session2 = std::make_shared<Session>();
+    EXPECT_THROW(multiperform.RemoveSession(session2), std::invalid_argument);
+}
+
 TEST(MultiperformGetTests, MultiperformSingleSessionGetTest) {
     Url url{server->GetBaseUrl() + "/hello.html"};
     std::shared_ptr<Session> session = std::make_shared<Session>();
@@ -183,6 +192,67 @@ TEST(MultiperformGetTests, MultiperformRemoveSessionGetTest) {
     EXPECT_EQ(ErrorCode::OK, responses.at(0).error.code);
 }
 
+TEST(MultiperformGetTests, MultiperformSingleSessionMultiGetTest) {
+    Url url{server->GetBaseUrl() + "/hello.html"};
+    std::shared_ptr<Session> session = std::make_shared<Session>();
+    session->SetUrl(url);
+    MultiPerform multiperform;
+    multiperform.AddSession(session);
+    std::vector<Response> responses = multiperform.Get();
+
+    EXPECT_EQ(responses.size(), 1);
+    std::string expected_text{"Hello world!"};
+    EXPECT_EQ(expected_text, responses.at(0).text);
+    EXPECT_EQ(url, responses.at(0).url);
+    EXPECT_EQ(std::string{"text/html"}, responses.at(0).header["content-type"]);
+    EXPECT_EQ(200, responses.at(0).status_code);
+    EXPECT_EQ(ErrorCode::OK, responses.at(0).error.code);
+
+    responses = multiperform.Get();
+    EXPECT_EQ(responses.size(), 1);
+    EXPECT_EQ(expected_text, responses.at(0).text);
+    EXPECT_EQ(url, responses.at(0).url);
+    EXPECT_EQ(std::string{"text/html"}, responses.at(0).header["content-type"]);
+    EXPECT_EQ(200, responses.at(0).status_code);
+    EXPECT_EQ(ErrorCode::OK, responses.at(0).error.code);
+}
+
+TEST(MultiperformGetTests, MultiperformAssignAfterUseTest) {
+    MultiPerform multiperform;
+    {
+        Url url{server->GetBaseUrl() + "/hello.html"};
+        std::shared_ptr<Session> session = std::make_shared<Session>();
+        session->SetUrl(url);
+        multiperform.AddSession(session);
+        std::vector<Response> responses = multiperform.Get();
+
+        EXPECT_EQ(responses.size(), 1);
+        std::string expected_text{"Hello world!"};
+        EXPECT_EQ(expected_text, responses.at(0).text);
+        EXPECT_EQ(url, responses.at(0).url);
+        EXPECT_EQ(std::string{"text/html"}, responses.at(0).header["content-type"]);
+        EXPECT_EQ(200, responses.at(0).status_code);
+        EXPECT_EQ(ErrorCode::OK, responses.at(0).error.code);
+    }
+
+    {
+        Url url{server->GetBaseUrl() + "/hello.html"};
+        std::shared_ptr<Session> session = std::make_shared<Session>();
+        session->SetUrl(url);
+        multiperform = MultiPerform();
+        multiperform.AddSession(session);
+        std::vector<Response> responses = multiperform.Get();
+
+        EXPECT_EQ(responses.size(), 1);
+        std::string expected_text{"Hello world!"};
+        EXPECT_EQ(expected_text, responses.at(0).text);
+        EXPECT_EQ(url, responses.at(0).url);
+        EXPECT_EQ(std::string{"text/html"}, responses.at(0).header["content-type"]);
+        EXPECT_EQ(200, responses.at(0).status_code);
+        EXPECT_EQ(ErrorCode::OK, responses.at(0).error.code);
+    }
+}
+
 #ifndef __APPLE__
 /**
  * This test case is currently disabled for macOS/Apple systems since it fails in an nondeterministic manner.
